(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react')) :
    typeof define === 'function' && define.amd ? define(['exports', 'react'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.ch = {}, global.React));
}(this, (function (exports, React) { 'use strict';

    function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

    var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */

    var __assign = function() {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };

    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m) return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
        }
        catch (error) { e = { error: error }; }
        finally {
            try {
                if (r && !r.done && (m = i["return"])) m.call(i);
            }
            finally { if (e) throw e.error; }
        }
        return ar;
    }

    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++)
            ar = ar.concat(__read(arguments[i]));
        return ar;
    }

    function useFrame(prevItems, nextItems, progress, padding) {
        var getFrame = React__default['default'].useMemo(function () { return getGetFrame(prevItems, nextItems, padding); }, [prevItems, nextItems, padding]);
        return getFrame(progress);
    }
    function getGetFrame(prevItems, nextItems, padding) {
        var currentItems = prevItems.filter(function (prev) {
            return prev.id != null &&
                nextItems.some(function (next) { return next.id === prev.id; });
        });
        var exitItems = prevItems.filter(function (prev) {
            return prev.id == null ||
                !nextItems.some(function (next) { return next.id === prev.id; });
        });
        var enterItems = nextItems.filter(function (next) {
            return next.id == null ||
                !prevItems.some(function (prev) { return prev.id === next.id; });
        });
        var stayPrevHeights = currentItems.map(function (item) { return item.height; });
        var stayNextHeights = currentItems.map(function (prev) {
            return nextItems.find(function (next) { return next.id === prev.id; }).height;
        });
        var exitHeights = exitItems.map(function (item) { return item.height; });
        var enterHeights = enterItems.map(function (item) { return item.height; });
        var prevTops = translates(__spread(exitHeights, stayPrevHeights), [], enterHeights, padding);
        var nextTops = translates(__spread(stayNextHeights, enterHeights), exitHeights, [], padding);
        var prevHeight = sum(prevItems.map(function (item) { return item.height; })) +
            prevItems.length * padding -
            padding;
        var nextHeight = sum(nextItems.map(function (item) { return item.height; })) +
            nextItems.length * padding -
            padding;
        var height = Math.min(prevHeight, nextHeight);
        var allItems = __spread(exitItems, currentItems, enterItems);
        var state = __spread(exitItems.map(function () { return "exit"; }), currentItems.map(function () { return "stay"; }), enterItems.map(function () { return "enter"; }));
        var heights = __spread(exitItems.map(function (_a) {
            var height = _a.height;
            return [height, height];
        }), currentItems.map(function (_a) {
            var id = _a.id;
            return [
                prevItems.find(function (item) { return item.id === id; }).height,
                nextItems.find(function (item) { return item.id === id; }).height,
            ];
        }), enterItems.map(function (_a) {
            var height = _a.height;
            return [height, height];
        }));
        return function (progress) {
            return {
                height: height,
                frame: allItems.map(function (item, i) { return ({
                    item: item,
                    itemHeight: tween(heights[i][0], heights[i][1], easing.easeInOutQuad(progress)),
                    translateY: tween(prevTops[i], nextTops[i], state[i] === "stay"
                        ? easing.easeInOutQuad(progress)
                        : state[i] === "exit"
                            ? easing.easeInQuad(progress)
                            : easing.easeOutQuad(progress)),
                    opacity: state[i] === "stay"
                        ? 1
                        : state[i] === "exit"
                            ? tweenOpacity(1 - progress)
                            : tweenOpacity(progress),
                }); }),
            };
        };
    }
    function tweenOpacity(t) {
        return Math.pow(t, 6);
    }
    function tween(p, n, t) {
        return (n - p) * t + p;
    }
    function translates(current, exit, enter, padding) {
        var total = sum(current) + (current.length - 1) * padding;
        var middle = total / 2;
        var acc = -middle;
        var currentTops = current.map(function (h) {
            var top = acc;
            acc += h + padding;
            return top;
        });
        var exitTotal = sum(exit) + exit.length * padding;
        acc = (currentTops[0] || -200) - exitTotal;
        var exitTops = exit.map(function (h) {
            var top = acc * 1.4;
            acc += h + padding;
            return top;
        });
        acc = (middle || 200) + padding;
        var enterTops = enter.map(function (h) {
            var top = acc * 1.4;
            acc += h + padding;
            return top;
        });
        return __spread(exitTops, currentTops, enterTops);
    }
    function sum(array) {
        return array.reduce(function (a, b) { return a + b; }, 0);
    }
    var easing = {
        linear: function (t) {
            return t;
        },
        easeInQuad: function (t) {
            return t * t;
        },
        easeOutQuad: function (t) {
            return t * (2 - t);
        },
        easeInOutQuad: function (t) {
            return t < 0.5
                ? 2 * t * t
                : 1 - Math.pow(-2 * t + 2, 2) / 2;
        },
    };

    function SmoothColumn(_a) {
        var _b, _c;
        var steps = _a.steps, padding = _a.padding, progress = _a.progress, _d = _a.style, style = _d === void 0 ? {} : _d, backward = _a.backward;
        var p = Math.min(Math.max(0, progress), steps.length - 1);
        var prevIndex = Math.floor(p);
        var nextIndex = prevIndex + 1;
        var prevItems = ((_b = steps[prevIndex]) === null || _b === void 0 ? void 0 : _b.items) || [];
        var nextItems = ((_c = steps[nextIndex]) === null || _c === void 0 ? void 0 : _c.items) || prevItems;
        var _e = useFrame(prevItems, nextItems, p % 1, padding), frame = _e.frame, height = _e.height;
        return (React__default['default'].createElement("div", { style: __assign({ height: height, position: "relative" }, style) }, frame.map(function (_a, i) {
            var item = _a.item, itemHeight = _a.itemHeight, translateY = _a.translateY, opacity = _a.opacity;
            return (React__default['default'].createElement("div", { style: {
                    position: "absolute",
                    top: "50%",
                    width: "100%",
                    height: item.height,
                    transform: "translateY(" + translateY + "px)",
                    opacity: opacity,
                }, key: item.id == null ? i : item.id }, React__default['default'].cloneElement(item.element, {
                progress: p,
                backward: backward,
                height: itemHeight,
            })));
        })));
    }

    exports.SmoothColumn = SmoothColumn;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
